//--------------------------------------------------------------------------
//
//	  Demo program to show the FT2XX drivers controlling
//	  the 8 bit port I/O. Written by Alex Drakard 2005.
//
//	  copyright stuff.... feel free to use it as you want.
//--------------------------------------------------------------------------

#include <cvirte.h>		
#include <userint.h>
#include <cviauto.h>
#include <ansi_c.h>
#include <formatio.h>
#include "FTDIPanel.h"

int panelmain;

extern int GetDevices(void);
extern int OpenDevice(int device_num);
extern int CloseDevice(void);
extern int GetDeviceInfo(void);
extern int IO_Setup(void);
extern int SetBitMode(UCHAR mask, UCHAR endis);
extern int GetBitMode(void);
extern int SetBit(int bit_num, int val);
extern int GetBit(int bit_num);

int UpdateDisplay(void);
int SetOutputs(int bit_num);

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelmain = LoadPanel (0, "FTDIPanel.uir", PANELMAIN)) < 0)
		return -1;
	GetDevices();
	DisplayPanel (panelmain);
	RunUserInterface ();
	DiscardPanel (panelmain);
	CloseDevice();
	return 0;
}

//--------------------------------------------------------------------------
int CVICALLBACK PanelCallbacks (int panel, int event, void *callbackData,
		int eventData1, int eventData2)
{
	if ((panel == panelmain) && (event == EVENT_CLOSE)) QuitUserInterface(0);

	return 0;
}

//--------------------------------------------------------------------------
int CVICALLBACK Callbacks (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	int val;
	int en;
	
	if ((panel == panelmain) && (event == EVENT_COMMIT))
		{
		if (control == PANELMAIN_CMDQUIT) QuitUserInterface(0);
		if (control == PANELMAIN_CMDREFRESH) GetDevices();
		if (control == PANELMAIN_CMDOPEN) 
			{
			GetCtrlIndex (panelmain, PANELMAIN_LSTDEVICES, &val);
			if (val<0) return -1;
			OpenDevice(val);
			}
		if (control == PANELMAIN_CMDCLOSE) CloseDevice();
		if (control == PANELMAIN_CMDINIT) IO_Setup();
		if (control == PANELMAIN_CMDGETINFO) GetDeviceInfo();
		if (control == PANELMAIN_CMDSETBITMODE) 
			{
			GetCtrlVal(panelmain, PANELMAIN_NUMSETBIT, &val);
			GetCtrlVal(panelmain, PANELMAIN_NUMENDIS, &en);
			SetBitMode(val, en);
			}
		if (control == PANELMAIN_CMDGETBITMODE) 
			{
			val = GetBitMode();
			SetCtrlVal(panelmain, PANELMAIN_NUMGETBIT, val);
			}
		if (control == PANELMAIN_BUTTON_0) SetOutputs(0);
		if (control == PANELMAIN_BUTTON_1) SetOutputs(1);
		if (control == PANELMAIN_BUTTON_2) SetOutputs(2);
		if (control == PANELMAIN_BUTTON_3) SetOutputs(3);
		if (control == PANELMAIN_BUTTON_4) SetOutputs(4);
		if (control == PANELMAIN_BUTTON_5) SetOutputs(5);
		if (control == PANELMAIN_BUTTON_6) SetOutputs(6);
		if (control == PANELMAIN_BUTTON_7) SetOutputs(7);
		}
	
	if ((panel == panelmain) && (control == PANELMAIN_TIMER) && (event == EVENT_TIMER_TICK)) 
		UpdateDisplay();
	return 0;
}


//--------------------------------------------------------------------------
int UpdateDisplay(void)
{
	int bit;
	
	bit = GetBit(0);
	SetCtrlVal(panelmain, PANELMAIN_LED_0, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_0, bit);
	bit = GetBit(1);
	SetCtrlVal(panelmain, PANELMAIN_LED_1, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_1, bit);
	bit = GetBit(2);
	SetCtrlVal(panelmain, PANELMAIN_LED_2, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_2, bit);
	bit = GetBit(3);
	SetCtrlVal(panelmain, PANELMAIN_LED_3, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_3, bit);
	bit = GetBit(4);
	SetCtrlVal(panelmain, PANELMAIN_LED_4, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_4, bit);
	bit = GetBit(5);
	SetCtrlVal(panelmain, PANELMAIN_LED_5, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_5, bit);
	bit = GetBit(6);
	SetCtrlVal(panelmain, PANELMAIN_LED_6, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_6, bit);
	bit = GetBit(7);
	SetCtrlVal(panelmain, PANELMAIN_LED_7, bit);
	SetCtrlVal(panelmain, PANELMAIN_BUTTON_7, bit);
	return 0;
}

//--------------------------------------------------------------------------
int SetOutputs(int bit_num)
{
	int bit;

	switch (bit_num)
		{
		case 0:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_0, &bit);
			SetBit(0,!bit);
			break;
		case 1:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_1, &bit);
			SetBit(1,!bit);
			break;
		case 2:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_2, &bit);
			SetBit(2,!bit);
			break;
		case 3:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_3, &bit);
			SetBit(3,!bit);
			break;
		case 4:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_4, &bit);
			SetBit(4,!bit);
			break;
		case 5:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_5, &bit);
			SetBit(5,!bit);
			break;
		case 6:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_6, &bit);
			SetBit(6,!bit);
			break;
		case 7:
			GetCtrlVal(panelmain, PANELMAIN_BUTTON_7, &bit);
			SetBit(7,!bit);
			break;
		}
	return 0;
}
